; View with TAB-size = 4 --------------------------------------------------
;==========================================================================
;       This program has been written for the microchip MPASM assembler 
;==========================================================================
;--------------------------------------------------------------------------
;                                                                          
; This bootloader requires the tiny bootloader, this version has been modified
; slightly in order to support the enable-bootloader jumper and to set the 'wait
; for bootloader pc software' for not 1 but for 5 seconds. This time-out is not
; used when the bootloader jumper is not placed so you do not notice this as a
; big delay.
; The bootloader is not protected, so when used wrong it CAN be overwritten!!!
;
; for more info about the wonderfull tiny bootloader
; contact	: claudiu.chiculita@ugal.ro   
; or visit	: http://www.etc.ugal.ro/cchiculita/software/picbootloader.htm                          
;                                                                          
; ------------------------------------------------------------------------ 
;==========================================================================
;
;ATTENTION: Loading of configuration registers using a bootlader can be dangerous...
;----------

;*************************************************************
; After reset
; Do not expect the memory to be zero,
; Do not expect registers to be initialised like in catalog.
;*************************************************************


;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                 - - = c o m p i l e r   o p t i o n s = - -          
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

	radix DEC

	LIST	P=18F2620		;microcontroller type

	;configuration register for 18F2620
	;----------------------------------
	CONFIG	OSC = HS		;set oscilator-mode
	CONFIG	FCMEN = OFF		;disbale failsafe clock monitor
	CONFIG	IESO = OFF		;disbale oscillator switchover mode
	CONFIG	PWRT = ON		;enable power-on timer
	CONFIG	BOREN = SBORDIS	;Enabled, SBOREN Disabled
	CONFIG	BORV = 2		;Brown-out Voltage (0=4.5V, 1=4.2V, 2=2.7V, 3=2.0V)
	CONFIG	WDT = OFF       ;watchdog disabled
	CONFIG	WDTPS = 32768	;prescaler set to 1:32768
	CONFIG  MCLRE = ON      ;enabled reset input 
	CONFIG	LPT1OSC = OFF	;T1 Oscillator Enable
	CONFIG	PBADEN = OFF	;PORTB<4:0> digital on Reset
	CONFIG	CCP2MX = PORTBE	;CCP2 MUX:Multiplexed with RB3
	CONFIG	STVREN = ON		;Stack Overflow Reset:Enabled	(if there is a bug in the software thataffects the stack, it is more likely to de detected this way. Better an unexpected reset then some fuzzy unexpected behaviour)
	CONFIG	LVP = OFF		;Low Voltage ICSP:Disabled
	CONFIG	XINST = OFF		;eXtended INSTruction set Enable:Disabled
	CONFIG	DEBUG = OFF		;Background Debugger Enable:Disabled
	CONFIG	CP0 = OFF		;Code Protection Block 0:Disabled
	CONFIG	CP1 = OFF		;Code Protection Block 1:Disabled
	CONFIG	CP2 = OFF		;Code Protection Block 2:Disabled
	CONFIG	CP3 = OFF		;Code Protection Block 3:Disabled
	CONFIG	CPB = OFF		;Boot Block Code Protection:Disabled
	CONFIG	CPD = OFF		;Data EEPROM Code Protection:Disabled
	CONFIG	WRT0 = OFF		;Write Protection Block 0:Disabled
	CONFIG	WRT1 = OFF		;Write Protection Block 1:Disabled
	CONFIG	WRT2 = OFF		;Write Protection Block 2:Disabled
	CONFIG	WRT3 = OFF		;Write Protection Block 3:Disabled
	CONFIG	EBTR0 = OFF		;Table Read Protection Block 0:Disabled
	CONFIG	EBTR1 = OFF		;Table Read Protection Block 1:Disabled
	CONFIG	EBTR2 = OFF		;Table Read Protection Block 2:Disabled
	CONFIG	EBTR3 = OFF		;Table Read Protection Block 3:Disabled
	CONFIG	EBTRB = OFF		;Boot Block Table Read Protection:Disabled
	CONFIG	WRTD = OFF		;Data EEPROM Write Protection:Disabled
	CONFIG	WRTB = OFF		;Boot Block Write Protection:Enabled
	CONFIG	WRTC = OFF		;Configuration Register Write Protection:Enabled

;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;             - - = v a r i a b l e    d e f i n i t i o n s = - -          
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

	cblock 0			;
	crc					;
	i					;
	cnt1				;
	cnt2				;
	cnt3				;
	counter_hi			;
	counter_lo			;
	flag				;
	endc				;
	cblock 10			;
	buffer:64			;
	endc				;


;==============================================================================
;                      - - =  C o n s t a n t s  = - - 
;==============================================================================

xtal 		EQU 	20000000			;
baud 		EQU 	115200				; the desired baud rate
timeouttime	EQU		5					;

#define first_address	max_flash-200	;100 words

;==============================================================================
;                      - - =  I n c l u d e s  = - - 
;==============================================================================

#include "icdpictypes.inc"		;takes care of: #include "p18fxxx.inc",  max_flash, IdTypePIC
#include "spbrgselect.inc"		;RoundResult and baud_rate

;==============================================================================
;                         - - =  M a c r o ' s  = - - 
;==============================================================================

SendL	macro car			;
			movlw car		;
			movwf TXREG		;
		endm				;
	
;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;                  - - = S T A R T   O F   C O D E = - -                         
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

	ORG     0x0000				;Start at the REAL reset vector
	GOTO    Bootloader			;

	ORG 	first_address		;space to deposit first 4 instr. of user prog.
	nop							;
	nop							;
	nop							;
	nop							;

	ORG 	first_address+8		;space for the real program, this is the startlocation of the interrupts

		
;//////////////////////////////////////////////////////////////////////////////
;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
;//////////////////////////////////////////////////////////////////////////////

;                                                                                              
;                       M     M    AA    III  N      N                      
;                       MM   MM   A  A    I   NN     N             
;                       M M M M  A    A   I   N N    N             
;                       M  M  M  A    A   I   N  N   N               
;                       M     M  AAAAAA   I   N   N  N                    
;                       M     M  A    A   I   N    N N                
;                       M     M  A    A   I   N     NN             
;                       M     M  A    A  III  N      N
;
;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
;//////////////////////////////////////////////////////////////////////////////
;//////////////////////////////////////////////////////////////////////////////
;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

;---------------------------------------------------------------------
;                               M A I N                               
;---------------------------------------------------------------------
	
Bootloader					;by default all I/O is input, so this saves us some trouble of setting these settings
	btfsc	PORTC,1			;test level is set(1) meaning jumper not placed, 
	bra 	first_address	;then execute program
							;else execute bootloader
	movlw b'00100100'		;init serial port
	movwf TXSTA				;
	movlw spbrg_value		;
	movwf SPBRG				;
	movlw b'10010000'		;
	movwf RCSTA				;
							;wait for computer
	rcall Receive			;
	sublw 0xC1				;Expect C1h
	bnz way_to_exit			;
	SendL IdTypePIC			;send PIC type

MainLoop
	SendL 'K'				; "-Everything OK, ready and waiting."
mainl
	clrf crc				;
	rcall Receive			;Upper
	movwf TBLPTRU			;
	movwf flag				;(for EEPROM and CFG cases)
	rcall Receive			;Hi
	movwf TBLPTRH			;
	movwf EEADR				;(for EEPROM case)
	rcall Receive			;Lo
	movwf TBLPTRL			;
	movwf EEDATA			;(for EEPROM case)

	rcall Receive			;count
	movwf i					;
	incf i					;
	lfsr FSR0, (buffer-1)	;
rcvoct						;read 64+1 bytes
	movwf TABLAT			;prepare for cfg; => store byte before crc
	rcall Receive			;
	movwf PREINC0			;
	decfsz i				;
	bra rcvoct				;
	
	tstfsz crc				;check crc
	bra ziieroare			;
	btfss flag,6			;is EEPROM data?
	bra noeeprom			;
	movlw b'00000100'		;Setup eeprom
	rcall Write				;
	bra waitwre				;
noeeprom
	btfss flag,7			;is CFG data?
	bra noconfig			;
	tblwt*					;write TABLAT(byte before crc) to TBLPTR***
	movlw b'11000100'		;Setup cfg
	rcall Write				;
	bra waitwre				;
noconfig
							;write
eraseloop
	movlw	b'10010100'		; Setup erase
	rcall Write				;
	TBLRD*-					; point to adr-1
	
writebigloop	
	movlw 8					; 8groups
	movwf counter_hi		;
	lfsr FSR0,buffer		;
writesloop
	movlw 8					; 8bytes = 4instr
	movwf counter_lo		;
writebyte
	movf POSTINC0,w			; put 1 byte
	movwf TABLAT			;
	tblwt+*					;
	decfsz counter_lo		;
	bra writebyte			;
	
	movlw	b'10000100'		; Setup writes
	rcall Write				;
	decfsz counter_hi		;
	bra writesloop			;
waitwre	
	;btfsc EECON1,WR		;for eeprom writes (wait to finish write)
	;bra waitwre			;no need: round trip time with PC bigger than 4ms
	
	bcf EECON1,WREN			;disable writes
	bra MainLoop			;
	
ziieroare					;CRC failed
	SendL 'N'				;
	bra mainl				;
	  
;******** procedures ******************

Write
	movwf EECON1			;
	movlw 0x55				;
	movwf EECON2			;
	movlw 0xAA				;
	movwf EECON2			;
	bsf EECON1,WR			;WRITE
	nop						;
	;nop					;
	return					;


Receive
	movlw timeouttime*(xtal/2000000+1)	; for 20MHz => 11 => 1second delay
	movwf cnt1				;
rpt2						
	clrf cnt2				;
rpt3
	clrf cnt3				;
rptc
	btfss PIR1,RCIF			;test RX
	bra notrcv				;
	movf RCREG,w			;return read data in W
	addwf crc,f				;compute crc
	return					;
notrcv
	decfsz cnt3				;
	bra rptc				;
	decfsz cnt2				;
	bra rpt3				;
	decfsz cnt1				;
	bra rpt2				;
							;timeout:
way_to_exit
	bcf RCSTA, SPEN			;deactivate UART
	bra first_address		;


	END
